from unittest import TestCase
from datetime import timedelta
from ready.checks.well_known import (
    check_robots_txt_exists,
    check_security_txt_exists,
    check_security_txt_not_expired,
    check_favicon_is_served,
    get_utc_time,
)
from ready.thttp import Response


class WellKnownChecksTestCase(TestCase):
    def test_check_robots_txt_exists(self):
        r = Response(None, b"", None, 200, None, {"content-type": "text/plain"}, None)
        result = check_robots_txt_exists({"robots_txt_response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, b"", None, 404, None, {}, None)
        result = check_robots_txt_exists({"robots_txt_response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_security_txt_exists(self):
        # Test with valid security.txt response
        security_response = Response(
            None,
            b"Contact: security@ready.invalid\nExpires: 2030-12-31T00:00:00Z",
            None,
            200,
            None,
            {"content-type": "text/plain"},
            None,
        )
        result = check_security_txt_exists({"security_txt_response": security_response}, print_output=False)
        self.assertTrue(result.passed)

        # Test with None security.txt response
        result = check_security_txt_exists({"security_txt_response": None}, print_output=False)
        self.assertFalse(result.passed)

        # Test with non-200 status code
        security_response = Response(None, b"", None, 404, None, {"content-type": "text/plain"}, None)
        result = check_security_txt_exists({"security_txt_response": security_response}, print_output=False)
        self.assertFalse(result.passed)

        # Test with incorrect content-type
        security_response = Response(None, b"", None, 200, None, {"content-type": "application/json"}, None)
        result = check_security_txt_exists({"security_txt_response": security_response}, print_output=False)
        self.assertFalse(result.passed)

        # Test with missing required attributes
        security_response = Response(
            None, b"Contact: security@ready.invalid", None, 200, None, {"content-type": "text/plain"}, None
        )
        result = check_security_txt_exists({"security_txt_response": security_response}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_favicon_is_served(self):
        # Test with valid favicon response
        favicon_response = Response(None, b"", None, 200, None, {"content-type": "image/x-icon"}, None)
        result = check_favicon_is_served({"favicon_response": favicon_response}, print_output=False)
        self.assertTrue(result.passed)

        # Test with invalid content-type
        favicon_response = Response(None, b"", None, 200, None, {"content-type": "text/html"}, None)
        result = check_favicon_is_served({"favicon_response": favicon_response}, print_output=False)
        self.assertFalse(result.passed)

        # Test with non-200 status code
        favicon_response = Response(None, b"", None, 404, None, {}, None)
        result = check_favicon_is_served({"favicon_response": favicon_response}, print_output=False)
        self.assertFalse(result.passed)

        # Test with None response
        result = check_favicon_is_served({"favicon_response": None}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_security_txt_not_expired(self):
        # Test with valid future expiry date
        future_date = get_utc_time() + timedelta(days=30)
        content = f"Expires: {future_date.isoformat()}\n".encode()
        security_response = Response(None, content, None, 200, {"content-type": "text/plain"}, {}, None)
        result = check_security_txt_not_expired({"security_txt_response": security_response}, print_output=False)
        self.assertTrue(result.passed)

        content = f"Expires: 2030-12-31T00:00:00Z\n".encode()
        security_response = Response(None, content, None, 200, {"content-type": "text/plain"}, {}, None)
        result = check_security_txt_not_expired({"security_txt_response": security_response}, print_output=False)
        self.assertTrue(result.passed)

        content = f"Expires: Not a date\n".encode()
        security_response = Response(None, content, None, 200, {"content-type": "text/plain"}, {}, None)
        result = check_security_txt_not_expired({"security_txt_response": security_response}, print_output=False)
        self.assertFalse(result.passed)

        # Test with expired expiry date
        past_date = get_utc_time() - timedelta(days=30)
        content = f"Expires: {past_date.isoformat()}\n".encode()
        security_response = Response(None, content, None, 200, {"content-type": "text/plain"}, {}, None)
        result = check_security_txt_not_expired({"security_txt_response": security_response}, print_output=False)
        self.assertFalse(result.passed)

        # Test with missing expiry date
        content = b"Contact: security@example.com\n"
        security_response = Response(None, content, None, 200, {"content-type": "text/plain"}, {}, None)
        result = check_security_txt_not_expired({"security_txt_response": security_response}, print_output=False)
        self.assertFalse(result.passed)



from unittest import TestCase

from ready.checks.status import check_http_response_should_be_200
from ready.thttp import Response


class StatusChecksTestCase(TestCase):
    def test_check_http_response_should_be_200(self):
        r = Response(None, "", None, 200, None, {}, None)
        result = check_http_response_should_be_200({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 499, None, {}, None)
        result = check_http_response_should_be_200({"response": r}, print_output=False)
        self.assertFalse(result.passed)



from unittest import TestCase, skipIf

from ready.checks.csp import (
    extract_csp,
    check_csp_should_exist,
    check_csp_should_start_with_defaultsrc_none,
    check_csp_includes_default_or_script_directive,
    check_csp_must_not_include_unsafe_eval,
    check_csp_must_not_include_unsafe_inline,
    check_csp_must_not_include_report_sample,
    check_csp_must_not_include_reporturi,
    check_csp_should_not_include_reportto,
    check_csp_upgrade_insecure_requests,
    check_csp_should_only_include_valid_directives,
)
from ready.thttp import Response

SKIP_BS4_TESTS = False
try:
    import bs4
except ImportError:
    SKIP_BS4_TESTS = True


response_with_html_csp = Response(
    None,
    """<!doctype html>
        <meta
          http-equiv="Content-Security-Policy"
          content="default-src 'self'; img-src https://*; child-src 'none';" />
    </head>""",
    None,
    None,
    None,
    {},
    None,
)


def response_with_csp(csp):
    return Response(None, "", None, None, None, {"content-security-policy": csp}, None)


class ExtractContentSecurityPolicyTestCase(TestCase):
    @skipIf(SKIP_BS4_TESTS, "beautifulsoup is not available")
    def test_extract_csp_meta_tag(self):
        from bs4 import BeautifulSoup

        csp = extract_csp(response_with_html_csp)
        self.assertEqual(csp, "default-src 'self'; img-src https://*; child-src 'none';")

    def test_extract_csp_from_header(self):
        csp = extract_csp(response_with_csp("default-src 'none';"))
        self.assertEqual(csp, "default-src 'none';")

    def test_extract_csp_empry(self):
        csp = extract_csp(Response(None, "", None, None, None, {}, None))
        self.assertEqual(csp, None)


class ContentSecurityPolicyChecksTestCase(TestCase):
    def test_check_csp_should_exist(self):
        result = check_csp_should_exist({"response": response_with_csp("default-src 'none';")}, print_output=False)
        self.assertTrue(result.passed)

        result = check_csp_should_exist({"response": Response(None, "", None, None, None, {}, None)}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_csp_should_start_with_defaultsrc_none(self):
        result = check_csp_should_start_with_defaultsrc_none(
            {"response": response_with_csp("default-src 'none';")}, print_output=False
        )
        self.assertTrue(result.passed)

        result = check_csp_should_start_with_defaultsrc_none(
            {"response": response_with_csp("default-src 'self';")}, print_output=False
        )
        self.assertFalse(result.passed)

    def test_check_csp_includes_default_or_script_directive(self):
        result = check_csp_includes_default_or_script_directive(
            {"response": response_with_csp("default-src 'none';")}, print_output=False
        )
        self.assertTrue(result.passed)

        result = check_csp_includes_default_or_script_directive(
            {"response": response_with_csp("script-src 'none';")}, print_output=False
        )
        self.assertTrue(result.passed)

        result = check_csp_includes_default_or_script_directive(
            {"response": response_with_csp("default-src 'none'; script-src 'self'")}, print_output=False
        )
        self.assertTrue(result.passed)

        result = check_csp_includes_default_or_script_directive(
            {"response": response_with_csp("upgrade-insecure-requests")}, print_output=False
        )
        self.assertFalse(result.passed)

    def test_check_csp_must_not_include_unsafe_eval(self):
        result = check_csp_must_not_include_unsafe_eval(
            {"response": response_with_csp("default-src 'none';")}, print_output=False
        )
        self.assertTrue(result.passed)

        result = check_csp_must_not_include_unsafe_eval(
            {"response": response_with_csp("default-src 'none'; unsafe-eval;")}, print_output=False
        )
        self.assertFalse(result.passed)

        result = check_csp_must_not_include_unsafe_eval(
            {"response": response_with_csp("default-src 'none'; UnSafe-EVAL")}, print_output=False
        )
        self.assertFalse(result.passed)

    def test_check_csp_must_not_include_unsafe_inline(self):
        result = check_csp_must_not_include_unsafe_inline(
            {"response": response_with_csp("default-src 'none';")}, print_output=False
        )
        self.assertTrue(result.passed)

        result = check_csp_must_not_include_unsafe_inline(
            {"response": response_with_csp("default-src 'none'; unsafe-inline;")}, print_output=False
        )
        self.assertFalse(result.passed)

        result = check_csp_must_not_include_unsafe_inline(
            {"response": response_with_csp("default-src 'none'; UnSafe-inLINe")}, print_output=False
        )
        self.assertFalse(result.passed)

    def test_check_csp_must_not_include_report_sample(self):
        result = check_csp_must_not_include_report_sample(
            {"response": response_with_csp("default-src 'none';")}, print_output=False
        )
        self.assertTrue(result.passed)

        result = check_csp_must_not_include_report_sample(
            {"response": response_with_csp("default-src 'none'; unsafe-inline; script-sample")}, print_output=False
        )
        self.assertFalse(result.passed)

    def test_check_csp_must_not_include_reporturi(self):
        result = check_csp_must_not_include_reporturi({"response": response_with_csp("default-src 'none';")}, print_output=False)
        self.assertTrue(result.passed)

        result = check_csp_must_not_include_reporturi(
            {"response": response_with_csp("default-src 'none'; report-uri https://example.org")}, print_output=False
        )
        self.assertFalse(result.passed)

    def test_check_csp_should_not_include_reportto(self):
        result = check_csp_should_not_include_reportto(
            {"response": response_with_csp("default-src 'none';")}, print_output=False
        )
        self.assertTrue(result.passed)

        result = check_csp_should_not_include_reportto(
            {"response": response_with_csp("default-src 'none'; report-to example")}, print_output=False
        )
        self.assertFalse(result.passed)

    def test_check_csp_upgrade_insecure_requests(self):
        result = check_csp_upgrade_insecure_requests(
            {"response": response_with_csp("default-src 'none'; upgrade-insecure-requests")}, print_output=False
        )
        self.assertTrue(result.passed)

        result = check_csp_upgrade_insecure_requests({"response": response_with_csp("default-src 'none'")}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_csp_should_only_include_valid_directives(self):
        result = check_csp_should_only_include_valid_directives(
            {"response": response_with_csp("default-src 'none'; upgrade-insecure-requests; invalid-directive;")},
            print_output=False,
        )
        self.assertFalse(result.passed)

    def test_checks_fail_with_missing_csp(self):
        checks = [
            check_csp_should_exist,
            check_csp_should_start_with_defaultsrc_none,
            check_csp_includes_default_or_script_directive,
            check_csp_upgrade_insecure_requests,
        ]

        for c in checks:
            result = c({"response": Response(None, "", None, None, None, {}, None)}, print_output=False)
            self.assertFalse(result.passed)

            result = c({"response": response_with_csp("")}, print_output=False)
            self.assertFalse(result.passed)

    def test_long_csp_should_be_truncated(self):
        result = check_csp_should_exist(
            {
                "response": response_with_csp(
                    "default-src 'none'; base-uri 'self'; child-src ready.invalid/assets-cdn/worker/ gist.ready.invalid/assets-cdn/worker/; connect-src 'self' uploads.ready.invalid collector.ready.invalid raw.ready.invalid api.ready.invalid  objects-origin.ready.invalid *.actions.ready.invalid wss://*.actions.ready.invalid insights.ready.invalid wss://alive.ready.invalid github.ready.invalid; font-src github.ready.invalid; form-action 'self' ready.invalid gist.ready.invalid objects-origin.ready.invalid; frame-ancestors 'none'; frame-src viewscreen.ready.invalid notebooks.ready.invalid; img-src 'self' data: github.ready.invalid media.ready.invalid camo.ready.invalid identicons.ready.invalid avatars.ready.invalid objects.ready.invalid secured-user-images.ready.invalid/ user-images.ready.invalid/ private-user-images.ready.invalid opengraph.ready.invalid customer-stories-feed.ready.invalid spotlights-feed.ready.invalid objects-origin.ready.invalid *.ready.invalid; manifest-src 'self'; media-src ready.invalid user-images.ready.invalid/ secured-user-images.ready.invalid/ private-user-images.ready.invalid github-production-user-asset-6210df.s3.amazonaws.com gist.ready.invalid github.ready.invalid; script-src github.ready.invalid; style-src 'unsafe-inline' github.ready.invalid; upgrade-insecure-requests; worker-src ready.invalid/assets-cdn/worker/ gist.ready.invalid/assets-cdn/worker/"
                )
            },
            print_output=False,
        )
        self.assertTrue("uploads.ready.invalid" in result.message)
        self.assertFalse("worker-src ready.invalid" in result.message)



from unittest import TestCase

from ready.checks.cookies import (
    check_cookies_should_be_samesite,
    check_cookies_should_be_secure,
    check_cookies_should_be_httponly,
)
from ready.thttp import Response


# logged_in=no; Path=/; Domain=github.com; Expires=Mon, 10 Mar 2025 21:34:03 GMT; HttpOnly; Secure; SameSite=Lax


class ContentChecksTestCase(TestCase):
    def test_check_cookies_should_be_samesite(self):
        r = Response(None, "", None, 200, None, {}, None)
        result = check_cookies_should_be_samesite({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {"set-cookie": "admin=no; Path=/; Domain=ready.invalid; SameSite=Lax"}, None)
        result = check_cookies_should_be_samesite({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {"set-cookie": "admin=no; Path=/; Domain=ready.invalid"}, None)
        result = check_cookies_should_be_samesite({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_cookies_should_be_secure(self):
        r = Response(None, "", None, 200, None, {}, None)
        result = check_cookies_should_be_secure({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {"set-cookie": "admin=no; Path=/; Domain=ready.invalid; Secure;"}, None)
        result = check_cookies_should_be_secure({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {"set-cookie": "admin=no; Path=/; Domain=ready.invalid;"}, None)
        result = check_cookies_should_be_secure({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_cookies_should_be_httponly(self):
        r = Response(None, "", None, 200, None, {}, None)
        result = check_cookies_should_be_httponly({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {"set-cookie": "admin=no; Path=/; Domain=ready.invalid; HttpOnly;"}, None)
        result = check_cookies_should_be_httponly({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {"set-cookie": "admin=no; Path=/; Domain=ready.invalid;"}, None)
        result = check_cookies_should_be_httponly({"response": r}, print_output=False)
        self.assertFalse(result.passed)



import os
from unittest import TestCase, skipIf

from ready.ready import ready

try:
    import bs4
    import tld
    import cryptography

    SKIP_READY_CHECKS = os.environ.get("READY_SKIP_E2E", "") == "1"
except ImportError as e:
    print(e)
    SKIP_READY_CHECKS = True


if SKIP_READY_CHECKS:
    print("Skipping end to end tests")


@skipIf(SKIP_READY_CHECKS, "Skipping because not all dependencies are available")
class ReadyTestCase(TestCase):
    def test_brntn(self):
        results = ready("brntn.me", hide_output=True)
        failures = [r.check for r in results if not r.passed]
        self.assertEqual(failures, ["ssl_dns_caa_accounturi", "ssl_dns_caa_validationmethods"])

    def test_basehtml(self):
        results = ready("basehtml.xyz", hide_output=True)

        failures = [r.check for r in results if not r.passed]

        self.assertEqual(
            failures,
            [
                "redirect_http",
                "ssl_hsts_preload",
                "csp_upgrade_insecure_requests",
                "csp_valid_directives",
                "report_to",
                "wellknown_robots",
                "wellknown_security",
                "wellknown_security_not_expired",
                "http_corp",
                "http_coop",
                "http_coep",
                "leaky_headers",
                "ssl_ocsp_must_staple",
                "ssl_dns_caa",
                "ssl_dns_caa_accounturi",
                "ssl_dns_caa_validationmethods",
                "email_dmarc_exists",
                "email_dmarc_none",
                "html_rel_icon",
                "html_unnecessary_entities",
                "html_x_dns_prefetch",
            ],
        )



from unittest import TestCase

from ready.checks.dns import check_aaaa_record_exists
from ready.thttp import Response


class DnsChecksTestCase(TestCase):
    def test_check_aaaa_record_exists(self):
        r = Response(None, "", {"Answer": [{"data": ""}]}, 200, None, {}, None)

        result = check_aaaa_record_exists({"dns_aaaa_response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", {"Answer": []}, 200, None, {}, None)

        result = check_aaaa_record_exists({"dns_aaaa_response": r}, print_output=False)
        self.assertFalse(result.passed)



from unittest import TestCase

from ready.checks.leaky_headers import check_should_not_include_leaky_headers
from ready.thttp import Response


class LeakyHeadersChecksTestCase(TestCase):
    def test_check_should_not_include_leaky_headers(self):
        r = Response(None, "", None, None, None, {}, None)
        result = check_should_not_include_leaky_headers({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, None, None, {"x-hosted-by": "22.11"}, None)
        result = check_should_not_include_leaky_headers({"response": r}, print_output=False)
        self.assertFalse(result.passed)



from unittest import TestCase

from ready.checks.redirect import check_http_to_https_redirect
from ready.thttp import Response


class RedirectChecksTestCase(TestCase):
    def test_check_http_to_https_redirect(self):
        r = Response(None, "", None, 200, "https://ready.invalid", {}, None)
        result = check_http_to_https_redirect({"http_response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, "http://ready.invalid", {}, None)
        result = check_http_to_https_redirect({"http_response": r}, print_output=False)
        self.assertFalse(result.passed)

        result = check_http_to_https_redirect({"http_response": None}, print_output=False)
        self.assertFalse(result.passed)



from unittest import TestCase

from ready.checks.corp_coop_coep import (
    check_cross_origin_resource_policy_should_be_sameorigin,
    check_cross_origin_opener_policy_should_be_sameorigin,
    check_cross_origin_embedder_policy_should_be_require_corp,
)
from ready.thttp import Response


class CorpCoopCoepChecksTestCase(TestCase):
    def test_check_cross_origin_resource_policy_should_be_sameorigin(self):
        r = Response(
            None,
            "",
            None,
            200,
            None,
            {
                "cross-origin-resource-policy": "same-origin",
            },
            None,
        )
        result = check_cross_origin_resource_policy_should_be_sameorigin({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {}, None)
        result = check_cross_origin_resource_policy_should_be_sameorigin({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_cross_origin_opener_policy_should_be_sameorigin(self):
        r = Response(
            None,
            "",
            None,
            200,
            None,
            {
                "cross-origin-opener-policy": "same-origin",
            },
            None,
        )
        result = check_cross_origin_opener_policy_should_be_sameorigin({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {}, None)
        result = check_cross_origin_opener_policy_should_be_sameorigin({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_cross_origin_embedder_policy_should_be_require_corp(self):
        r = Response(
            None,
            "",
            None,
            200,
            None,
            {
                "cross-origin-embedder-policy": "require-corp",
            },
            None,
        )
        result = check_cross_origin_embedder_policy_should_be_require_corp({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {}, None)
        result = check_cross_origin_embedder_policy_should_be_require_corp({"response": r}, print_output=False)
        self.assertFalse(result.passed)



from unittest import TestCase

from ready.checks.hsts import (
    check_hsts_header_should_be_included_in_response,
    check_hsts_header_should_have_a_long_max_age,
    check_hsts_header_should_have_includesubdomains,
    check_hsts_header_should_have_preload,
)
from ready.thttp import Response


class HstsChecksTestCase(TestCase):
    def test_check_hsts_header_should_be_included_in_response(self):
        r = Response(None, "", None, 200, None, {"strict-transport-security": "max-age=0"}, None)
        result = check_hsts_header_should_be_included_in_response({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {}, None)
        result = check_hsts_header_should_be_included_in_response({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_hsts_header_should_have_a_long_max_age(self):
        r = Response(None, "", None, 200, None, {"strict-transport-security": "max-age=31536001"}, None)
        result = check_hsts_header_should_have_a_long_max_age({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {"strict-transport-security": "max-age=0"}, None)
        result = check_hsts_header_should_have_a_long_max_age({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_hsts_header_should_have_includesubdomains(self):
        r = Response(None, "", None, 200, None, {"strict-transport-security": "max-age=0; includeSubDomains"}, None)
        result = check_hsts_header_should_have_includesubdomains({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {"strict-transport-security": "max-age=0;"}, None)
        r_fld = Response(None, "", None, 200, None, {"strict-transport-security": "max-age=0; includeSubDomains"}, None)
        result = check_hsts_header_should_have_includesubdomains({"response": r, "response_fld": r_fld}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {}, None)
        r_fld = Response(None, "", None, 200, None, {"strict-transport-security": "max-age=0; includeSubDomains"}, None)
        result = check_hsts_header_should_have_includesubdomains({"response": r, "response_fld": r_fld}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {}, None)
        result = check_hsts_header_should_have_includesubdomains({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_hsts_header_should_have_preload(self):
        r = Response(None, "", None, 200, None, {"strict-transport-security": "max-age=0; preload; includeSubDomains"}, None)
        result = check_hsts_header_should_have_preload({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {"strict-transport-security": "max-age=0;"}, None)
        r_fld = Response(None, "", None, 200, None, {"strict-transport-security": "max-age=0; preload; includeSubDomains"}, None)
        result = check_hsts_header_should_have_preload({"response": r, "response_fld": r_fld}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {}, None)
        r_fld = Response(None, "", None, 200, None, {"strict-transport-security": "max-age=0; preload; includeSubDomains"}, None)
        result = check_hsts_header_should_have_preload({"response": r, "response_fld": r_fld}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {}, None)
        result = check_hsts_header_should_have_preload({"response": r}, print_output=False)
        self.assertFalse(result.passed)



from unittest import TestCase, skipIf
from unittest.mock import patch
from ready.checks.html import (
    check_permissions_policy_should_exist,
    check_frame_ancestors_should_exist,
    check_x_content_type_options_should_be_nosniff,
    check_referrer_policy_should_be_set,
    check_x_xss_protection_should_not_exist,
    check_html_starts_with_doctype,
    check_html_tag_includes_lang,
    check_html_meta_charset,
    check_html_includes_title,
    check_html_includes_rel_icon,
    check_html_should_not_use_schemeless_urls,
    check_html_should_not_use_unnecessary_entities,
    check_html_script_tags_use_sri,
    check_x_dns_prefetch_control_is_off,
    check_cdns_should_not_be_used,
    check_rss_should_return_cors_header,
    check_html_should_not_be_cached_for_more_than_24_hours,
)

from ready.thttp import Response

SKIP_BS4_TESTS = False
try:
    import bs4
except ImportError:
    SKIP_BS4_TESTS = True


class HtmlChecksTestCase(TestCase):
    def test_check_permissions_policy_should_exist(self):
        r = Response(None, "", None, 200, None, {"permissions-policy": "camera=()"}, None)
        result = check_permissions_policy_should_exist({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {}, None)
        result = check_permissions_policy_should_exist({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_frame_ancestors_should_exist(self):
        r = Response(None, "", None, 200, None, {"x-frame-options": "DENY"}, None)
        result = check_frame_ancestors_should_exist({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {"content-security-policy": "frame-ancestors 'none'"}, None)
        result = check_frame_ancestors_should_exist({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {}, None)
        result = check_frame_ancestors_should_exist({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_x_content_type_options_should_be_nosniff(self):
        r = Response(None, "", None, 200, None, {"x-content-type-options": "nosniff"}, None)
        result = check_x_content_type_options_should_be_nosniff({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {"x-content-type-options": "other-value"}, None)
        result = check_x_content_type_options_should_be_nosniff({"response": r}, print_output=False)
        self.assertFalse(result.passed)

        r = Response(None, "", None, 200, None, {}, None)
        result = check_x_content_type_options_should_be_nosniff({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_referrer_policy_should_be_set(self):
        r = Response(None, "", None, 200, None, {"referrer-policy": "no-referrer"}, None)
        result = check_referrer_policy_should_be_set({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {}, None)
        result = check_referrer_policy_should_be_set({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_x_xss_protection_should_not_exist(self):
        r = Response(None, "", None, 200, None, {}, None)
        result = check_x_xss_protection_should_not_exist({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {"x-xss-protection": "some-value"}, None)
        result = check_x_xss_protection_should_not_exist({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_html_starts_with_doctype(self):
        r = Response(None, b"    <!doctype html><meta description='test' />", None, 200, None, {}, None)
        result = check_html_starts_with_doctype({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, b"aaaa<!doctype html><meta description='test' />", None, 200, None, {}, None)
        result = check_html_starts_with_doctype({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_html_tag_includes_lang(self):
        r = Response(None, b"<html js lang='en'>", None, 200, None, {}, None)
        result = check_html_tag_includes_lang({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, b"<html js>", None, 200, None, {}, None)
        result = check_html_tag_includes_lang({"response": r}, print_output=False)
        self.assertFalse(result.passed)

        r = Response(None, b"", None, 200, None, {}, None)
        result = check_html_tag_includes_lang({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_html_meta_charset(self):
        r = Response(None, b"<html js lang='en'><meta charset='utf-8'>", None, 200, None, {}, None)
        result = check_html_meta_charset({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, b"", None, 200, None, {}, None)
        result = check_html_meta_charset({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_html_includes_title(self):
        r = Response(None, b"<html js lang='en'><title>Test</title>", None, 200, None, {}, None)
        result = check_html_includes_title({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, b"", None, 200, None, {}, None)
        result = check_html_includes_title({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_html_includes_rel_icon(self):
        r = Response(None, b"<link rel='icon' href='favicon.ico'>", None, 200, None, {}, None)
        result = check_html_includes_rel_icon({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, b"<link href='favicon.ico' rel='icon'>", None, 200, None, {}, None)
        result = check_html_includes_rel_icon({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, b"<link rel='shortcut icon' href='favicon.ico'>", None, 200, None, {}, None)
        result = check_html_includes_rel_icon({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, b"<link href='favicon.ico' rel='shortcut icon'>", None, 200, None, {}, None)
        result = check_html_includes_rel_icon({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, b"", None, 200, None, {}, None)
        result = check_html_includes_rel_icon({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_html_should_not_use_schemeless_urls(self):
        r = Response(None, b"<html>", None, 200, None, {}, None)
        result = check_html_should_not_use_schemeless_urls({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, b"='//'", None, 200, None, {}, None)
        result = check_html_should_not_use_schemeless_urls({"response": r}, print_output=False)
        self.assertFalse(result.passed)

        r = Response(None, b'="//"', None, 200, None, {}, None)
        result = check_html_should_not_use_schemeless_urls({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_html_should_not_use_unnecessary_entities(self):
        r = Response(None, b"<html>", None, 200, None, {}, None)
        result = check_html_should_not_use_unnecessary_entities({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, b"&nbsp;", None, 200, None, {}, None)
        result = check_html_should_not_use_unnecessary_entities({"response": r}, print_output=False)
        self.assertFalse(result.passed)

        r = Response(None, b"&nbsp;&amp;&quot;", None, 200, None, {}, None)
        result = check_html_should_not_use_unnecessary_entities({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_html_script_tags_use_sri(self):
        r = Response(None, b'<script src="example.js" integrity="sha256-abc123"></script>', None, 200, None, {}, None)
        result = check_html_script_tags_use_sri({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, b'<script src="example.js"></script>', None, 200, None, {}, None)
        result = check_html_script_tags_use_sri({"response": r}, print_output=False)
        self.assertFalse(result.passed)

        r = Response(
            None,
            b'<script src="example1.js"></script><script src="example2.js" integrity="sha256-abc123"></script>',
            None,
            200,
            None,
            {},
            None,
        )
        result = check_html_script_tags_use_sri({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_x_dns_prefetch_control_is_off(self):
        r = Response(None, b"", None, 200, None, {"x-dns-prefetch-control": "off"}, None)
        result = check_x_dns_prefetch_control_is_off({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, b"", None, 200, None, {"x-dns-prefetch-control": "on"}, None)
        result = check_x_dns_prefetch_control_is_off({"response": r}, print_output=False)
        self.assertFalse(result.passed)

        r = Response(None, b"", None, 200, None, {}, None)
        result = check_x_dns_prefetch_control_is_off({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_cdns_should_not_be_used(self):
        r = Response(
            None, b'<script src="local.js"></script><link rel="stylesheet" href="local.css">', None, 200, None, {}, None
        )
        result = check_cdns_should_not_be_used({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, b'<script src="https://cdn.jsdelivr.net/example.js"></script>', None, 200, None, {}, None)
        result = check_cdns_should_not_be_used({"response": r}, print_output=False)
        self.assertFalse(result.passed)

        r = Response(None, b'<link rel="stylesheet" href="https://cdnjs.cloudflare.com/example.css">', None, 200, None, {}, None)
        result = check_cdns_should_not_be_used({"response": r}, print_output=False)
        self.assertFalse(result.passed)

        r = Response(
            None,
            b'<script src="local.js"></script><link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/4.0.0/css/bootstrap.min.css">',
            None,
            200,
            None,
            {},
            None,
        )
        result = check_cdns_should_not_be_used({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_html_should_not_be_cached_for_more_than_24_hours(self):
        r = Response(None, b"", None, 200, None, {"cache-control": "max-age=86400"}, None)
        result = check_html_should_not_be_cached_for_more_than_24_hours({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, b"", None, 200, None, {"cache-control": "max-age=90000"}, None)
        result = check_html_should_not_be_cached_for_more_than_24_hours({"response": r}, print_output=False)
        self.assertFalse(result.passed)

        r = Response(None, b"", None, 200, None, {}, None)
        result = check_html_should_not_be_cached_for_more_than_24_hours({"response": r}, print_output=False)
        self.assertFalse(result.passed)

        r = Response(None, b"", None, 200, None, {"cache-control": "max-age=abc"}, None)
        result = check_html_should_not_be_cached_for_more_than_24_hours({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    @skipIf(SKIP_BS4_TESTS, "beautifulsoup is not available")
    def test_check_rss_should_return_cors_header(self):
        mock_response = Response(None, b"", None, 200, None, {"access-control-allow-origin": "*"}, None)
        with patch("ready.checks.html.thttp.request", return_value=mock_response):
            r = Response(
                None,
                b'<link rel="alternate" type="application/rss+xml" href="https://thttp.invalid/feed.rss">',
                None,
                200,
                "https://ready.invalid",
                {},
                None,
            )
            result = check_rss_should_return_cors_header({"response": r}, print_output=False)
            self.assertTrue(result.passed)

            r = Response(
                None,
                b'<link rel="alternate" type="application/rss+xml" href="//thttp.invalid/feed.rss">',
                None,
                200,
                "https://ready.invalid",
                {},
                None,
            )
            result = check_rss_should_return_cors_header({"response": r}, print_output=False)
            self.assertTrue(result.passed)

            r = Response(
                None,
                b'<link rel="alternate" type="application/rss+xml" href="/feed.rss">',
                None,
                200,
                "https://ready.invalid",
                {},
                None,
            )
            result = check_rss_should_return_cors_header({"response": r}, print_output=False)
            self.assertTrue(result.passed)

        mock_response = Response(None, b"", None, 200, None, {}, None)
        with patch("ready.checks.html.thttp.request", return_value=mock_response):
            r = Response(
                None,
                b'<link rel="alternate" type="application/rss+xml" href="https://thttp.invalid/feed.rss">',
                None,
                200,
                {},
                {},
                None,
            )
            result = check_rss_should_return_cors_header({"response": r}, print_output=False)
            self.assertFalse(result.passed)



from ready.checks.swagger import check_swagger_should_not_return_200, SWAGGER_PATHS
from unittest import TestCase


from ready.thttp import Response
from unittest.mock import patch


class SwaggerChecksTestCase(TestCase):
    def test_check_swagger_should_not_return_200(self):
        mocked_response = Response(None, "", None, 404, None, None, None)
        r = Response(None, "", None, 200, "https://ready.invalid", {}, None)

        with patch("ready.checks.swagger.request", return_value=mocked_response):
            result = check_swagger_should_not_return_200({"response": r}, print_output=False)
            self.assertTrue(result.passed)

        mocked_response = Response(None, "", None, 200, None, None, None)
        with patch("ready.checks.swagger.request", return_value=mocked_response):
            result = check_swagger_should_not_return_200({"response": r}, print_output=False)
            self.assertFalse(result.passed)



from unittest import TestCase

from ready.checks.report_to import check_report_to_header_must_not_be_included_in_response
from ready.thttp import Response


class ReportToChecksTestCase(TestCase):
    def test_check_report_to_header_must_not_be_included_in_response(self):
        r = Response(None, "", None, None, None, {}, None)
        result = check_report_to_header_must_not_be_included_in_response({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, None, None, {"report-to": "some-value"}, None)
        result = check_report_to_header_must_not_be_included_in_response({"response": r}, print_output=False)
        self.assertFalse(result.passed)



from unittest import TestCase

from ready.checks.content import (
    check_http_response_should_include_content_type,
    check_http_response_should_be_gzipped,
    check_http_content_type_header_contains_charset,
    check_http_cache_control_is_included,
    check_http_p3p_header_is_not_set,
    check_http_expires_header_not_used_without_cache_control,
)
from ready.thttp import Response


class ContentChecksTestCase(TestCase):
    def test_check_http_response_should_include_content_type(self):
        r = Response(None, "", None, 200, None, {"content-type": "example/example"}, None)
        result = check_http_response_should_include_content_type({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {}, None)
        result = check_http_response_should_include_content_type({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_http_response_should_be_gzipped(self):
        r = Response(None, "", None, 200, None, {"content-encoding": "gzip; example"}, None)
        result = check_http_response_should_be_gzipped({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {}, None)
        result = check_http_response_should_be_gzipped({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_http_content_type_header_contains_charset(self):
        r = Response(None, "", None, 200, None, {"content-type": "example/example; charset=utf-8"}, None)
        result = check_http_content_type_header_contains_charset({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {"content-type": "example/example"}, None)
        result = check_http_content_type_header_contains_charset({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_http_cache_control_is_included(self):
        r = Response(None, "", None, 200, None, {"cache-control": "nocache"}, None)
        result = check_http_cache_control_is_included({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {}, None)
        result = check_http_cache_control_is_included({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_http_p3p_header_is_not_set(self):
        r = Response(None, "", None, 200, None, {}, None)
        result = check_http_p3p_header_is_not_set({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {"p3p": "some-value"}, None)
        result = check_http_p3p_header_is_not_set({"response": r}, print_output=False)
        self.assertFalse(result.passed)

    def test_check_http_expires_header_not_used_without_cache_control(self):
        r = Response(None, "", None, 200, None, {"cache-control": "maxage=20"}, None)
        result = check_http_expires_header_not_used_without_cache_control({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {"cache-control": "maxage=20", "expires": "Wed, 21 Oct 2015 07:28:00 GMT"}, None)
        result = check_http_expires_header_not_used_without_cache_control({"response": r}, print_output=False)
        self.assertTrue(result.passed)

        r = Response(None, "", None, 200, None, {"expires": "Wed, 21 Oct 2015 07:28:00 GMT"}, None)
        result = check_http_expires_header_not_used_without_cache_control({"response": r}, print_output=False)
        self.assertFalse(result.passed)
